@@ -64,6 +64,10 @@ void MainWindow::setupActions() {
     m_action_pause = KStandardGameAction::pause(this, SLOT(togglePaused(bool)), actionCollection());
     m_action_undo = KStandardGameAction::undo(this, SLOT(undo()), actionCollection());
     m_action_hint = KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
+    m_action_solve = KStandardGameAction::solve(this, SLOT(solve()), actionCollection());
+
+    /* Prevent the default hint shortcut from overwriting our HJKL vim-like control mapping. */
+    m_action_hint->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_I));
 
     m_action_save_state = actionCollection()->addAction("save-position");
     m_action_save_state->setText(i18n("Save Position"));
@@ -162,8 +166,16 @@ void MainWindow::undo() {
 void MainWindow::hint()
 {
     QPoint p = m_game->hint();
-    m_scene->refresh(p);
-    m_scene->hover(p.x(), p.y());
+
+    if (m_in_progress) {
+        m_scene->refresh(p);
+        m_scene->hover(p.x(), p.y());
+    }
+}
+
+void MainWindow::solve()
+{
+    m_game->solve();
 }
 
 void MainWindow::saveState() {
@@ -207,6 +219,7 @@ void MainWindow::startGame() {
 
     m_action_undo->setEnabled(false);
     m_action_hint->setEnabled(true);
+    m_action_solve->setEnabled(true);
     m_action_save_state->setEnabled(true);
     m_action_load_state->setEnabled(false);
     m_action_pause->setEnabled(true);
@@ -224,6 +237,7 @@ void MainWindow::startGame() {
 
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(updatePlayedTime()));
     connect(m_game.data(), SIGNAL(stateChanged()), this, SLOT(updatePositions()));
+    connect(m_game.data(), SIGNAL(gameCompleted()), this, SLOT(gameCompleted()));
     connect(m_game.data(), SIGNAL(gameWon()), this, SLOT(gameWon()));
     connect(m_game.data(), SIGNAL(undoStackSizeChanged(int)), this, SLOT(undoStackSizeChanged(int)));
     connect(m_game.data(), SIGNAL(saveStackSizeChanged(int)), this, SLOT(saveStackSizeChanged(int)));
@@ -252,16 +266,6 @@ QSharedPointer<KScoreDialog> MainWindow::createScoreDialog() {
 
 void MainWindow::gameWon() {
     KScoreDialog::FieldInfo score = m_game->endGame();
-    m_view.setEnabled(false);
-    m_action_pause->setEnabled(false);
-    m_action_hint->setEnabled(false);
-    m_action_undo->setEnabled(false);
-    m_action_save_state->setEnabled(false);
-    m_action_load_state->setEnabled(false);
-    Kg::difficulty()->setGameRunning(false);
-    m_timer.stop();
-    m_in_progress = false;
-
     bool notified = false;
     if (m_mode == Random && Kg::difficultyLevel() != KgDifficultyLevel::Custom) {
         QSharedPointer<KScoreDialog> scoreDialog = createScoreDialog();
@@ -282,6 +286,19 @@ void MainWindow::gameWon() {
     m_view.setFocus();
 }
 
+void MainWindow::gameCompleted() {
+    m_view.setEnabled(false);
+    m_action_pause->setEnabled(false);
+    m_action_solve->setEnabled(false);
+    m_action_hint->setEnabled(false);
+    m_action_undo->setEnabled(false);
+    m_action_save_state->setEnabled(false);
+    m_action_load_state->setEnabled(false);
+    Kg::difficulty()->setGameRunning(false);
+    m_timer.stop();
+    m_in_progress = false;
+}
+
 void MainWindow::highscores() {
     pauseGame();
 
@@ -295,6 +312,7 @@ void MainWindow::togglePaused(bool paused) {
     m_view.setPaused(paused);
     m_action_undo->setEnabled(!paused);
     m_action_hint->setEnabled(!paused);
+    m_action_solve->setEnabled(!paused);
     m_action_save_state->setEnabled(!paused);
     m_action_load_state->setEnabled(!paused);
 
